﻿using System;
using System.Linq;
using OnYourWayHome.ApplicationModel;
using OnYourWayHome.ApplicationModel.Composition;
using OnYourWayHome.ApplicationModel.Presentation.Navigation;
using OnYourWayHome.ViewModels;
using OnYourWayHome.ServiceBus;
using Autofac;
using System.Collections.Generic;
using System.Reflection;

namespace OnYourWayHome
{
    // Represents the shared application object between device-specific 'Application' objects
    public class OnYourWayHomeApplication : DisposableObject, IResolver
    {
        private readonly IContainer _container;

        public OnYourWayHomeApplication(Type hostType)
        {
            Requires.NotNull(hostType, "hostType");

            var assemblies = new[] { typeof(OnYourWayHomeApplication).Assembly,
                                     hostType.Assembly };

            var builder = new ContainerBuilder();

            builder.RegisterAssemblyTypes(assemblies)
                .SingleInstance()
                .As(t => ServiceTypes(t));
            
            builder.RegisterInstance(this).As<IResolver>();

            _container = builder.Build();
        }

        private IEnumerable<Type> ServiceTypes(Type t)
        {
            foreach (Type type in t.GetInterfaces())
                yield return type;

            if (t.BaseType != typeof(object))
                yield return t.BaseType;

            yield return t;
        }

        public void Start()
        {
            // Create all the startup services
            _container.Resolve<IEnumerable<IStartupService>>();

            INavigationService navigation = _container.Resolve<INavigationService>();
            navigation.NavigateTo<ShoppingListViewModel>();
        }

        public object Resolve(Type type)
        {
            return _container.Resolve(type);
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing)
            {
                _container.Dispose();
            }
        }
    }
}
