﻿using System;
using System.Threading;
using OnYourWayHome.ServiceBus.Messaging;

namespace OnYourWayHome.ServiceBus
{
    // Polls an Azure ServiceBus subscription on a background thread looking for messages
    internal class SubscriptionClientPoller : IDisposable
    {
        private readonly ISubscriptionClient _subscription;
        private readonly Action<BrokeredMessage> _callback;
        private readonly Timer _timer;

        public SubscriptionClientPoller(ISubscriptionClient subscription, Action<BrokeredMessage> callback)
        {
            Assumes.NotNull(subscription);
            Assumes.NotNull(callback);

            _subscription = subscription;
            _callback = callback;
            _timer = new Timer(OnTimerElapsed, (object)null, 0, Timeout.Infinite);
        }

        public void Dispose()
        {
            _timer.Dispose();
        }

        private void Sleep()
        {
            _timer.Change(5000, Timeout.Infinite);
        }

        private void OnTimerElapsed(object state)
        {   // Executed on a ThreadPool thread
            
            BrokeredMessage message;
            while ((message = Receive()) != null)
            {
                _callback(message);
            }

            // No more messages, sleep
            Sleep();
        }

        private BrokeredMessage Receive()
        {
            IAsyncResult result = _subscription.BeginReceive((AsyncCallback)null);

            return _subscription.EndReceive(result);
        }
    }
}
